var Crypto, Fs, Mime, Path, Util, empty_cb;
Util = require('util');
Path = require('path');
Fs = require('fs');
Crypto = require('crypto');
Mime = require('simple-mime')('application/octect-stream');
module.exports = function(root, options) {
  var defaultFile;
  if (options == null) {
    options = {};
  }
  root = Path.normalize(root);
  defaultFile = options["default"];
  delete options["default"];
  if (!(options.etag != null)) {
    options.etag = module.exports.etag;
  }
  return function(req, res, next) {
    if (defaultFile && req.url.match(/\/$/)) {
      req.url = "" + req.url + defaultFile;
    }
    options.file = "" + root + req.url;
    return module.exports.serve(req, res, options, function() {
      return next();
    });
  };
};
module.exports.serve = function(req, res, options, next) {
  var file;
  if (options == null) {
    options = {};
  }
  file = options.file;
  delete options.file;
  return Fs.stat(file, function(err, stats) {
    if (err) {
      if (next) {
        next(err);
      }
      return;
    }
    return module.exports.writeHead(req, res, file, stats, options, function(status) {
      return Util.pump(Fs.createReadStream(file), res, next || empty_cb);
    });
  });
};
empty_cb = function() {};
module.exports.writeHead = function(req, res, file, stats, options, cb) {
  var headers, status;
  if (options == null) {
    options = {};
  }
  headers = module.exports.defaultHeaders(file, stats);
  status = options.status || 200;
  if (options.etag) {
    return options.etag(file, stats, function(etag) {
      headers.etag = etag;
      res.writeHead(status, headers);
      return cb();
    });
  } else {
    res.writeHead(status, headers);
    return cb();
  }
};
module.exports.defaultHeaders = function(file, stats) {
  return {
    'content-type': Mime(file),
    'content-length': stats.size,
    'last-modified': stats.mtime.toUTCString()
  };
};
module.exports.etag = function(file, stats, next) {
  var hash;
  hash = Crypto.createHash('sha1');
  hash.update(stats.ino);
  hash.update(stats.size);
  hash.update(stats.mtime);
  return next("\"" + (hash.digest('hex')) + "\"");
};